package org.hso.ecommerce.controller.shop;

import org.hso.ecommerce.entities.shop.Article;
import org.hso.ecommerce.entities.shop.ShoppingCart;
import org.hso.ecommerce.repos.shop.ArticleRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestAttribute;
import org.springframework.web.bind.annotation.RequestMapping;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.util.ArrayList;
import java.util.TreeMap;

@Controller
@RequestMapping("/shop/")
public class ShopCheckoutController {

    @Autowired
    private final ArticleRepository articleRepository = null;

    @GetMapping("/checkout")
    public String shopCheckout(HttpSession session,
                               HttpServletRequest request,
                               @RequestAttribute(value = "shoppingCart") ShoppingCart shoppingCart) {
        session.setAttribute("afterLogin", request.getRequestURI());

        CheckoutListTotals totals = new CheckoutListTotals();
        ArrayList<CheckoutListItem> items = new ArrayList<>();
        for (ShoppingCart.ShoppingCartItem item : shoppingCart.getItems()) {
            Article article = articleRepository.findById(item.getArticleId()).get();

            totals.addItem(article, item.getAmount());
            items.add(new CheckoutListItem(item.getAmount(), article));
        }

        request.setAttribute("checkoutItems", items);
        request.setAttribute("checkoutTotals", totals);

        return "shop/checkout";
    }

    public static class CheckoutListTotals {
        public int net;
        public TreeMap<Integer, Integer> vatAmounts = new TreeMap<>();
        public int total;

        void addItem(Article article, int amount) {
            net += article.shopPricePerUnitNetCent * amount;
            Integer vatPos = vatAmounts.getOrDefault(article.related.vatPercent, 0) + article.getVat() * amount;
            vatAmounts.put(article.related.vatPercent, vatPos);
            total += article.getPriceGross() * amount;
        }
    }

    public static class CheckoutListItem {
        public int amount;
        public Article article;
        public int total;

        public CheckoutListItem(int amount, Article article) {
            this.amount = amount;
            this.article = article;
            this.total = amount * article.getPriceGross();
        }
    }

    @PostMapping("/checkoutFinish")
    public String shopCheckoutFinish() {
        return "shop/checkoutFinish";
    }

    @GetMapping("/checkoutFinish")
    public String shopCheckoutFinishGET() {
        return "shop/checkoutFinish";
    }
}
